@@ -9,6 +9,7 @@ extensions = [
     "sphinx.ext.autodoc",
     "sphinx.ext.intersphinx",
     "sphinx_autodoc_typehints",
+    "sphinx_rtd_theme",
 ]
 
 templates_path = ["_templates"]
@@ -27,10 +28,10 @@ language = "en"
 exclude_patterns = ["_build"]
 pygments_style = "sphinx"
 autodoc_default_options = {"members": True, "show-inheritance": True}
-autodoc_mock_imports = ["_typeshed"]
+autodoc_mock_imports = ["_typeshed", "pytest", "_pytest"]
 todo_include_todos = False
 
-html_theme = "nature"
+html_theme = "sphinx_rtd_theme"
 htmlhelp_basename = "anyiodoc"
 
 intersphinx_mapping = {"python": ("https://docs.python.org/3/", None)}
